#tag Window
Begin Window Main
   BackColor       =   &cFFFFFF00
   Backdrop        =   0
   BalloonHelp     =   ""
   CloseButton     =   True
   Compatibility   =   ""
   Composite       =   False
   Frame           =   0
   FullScreen      =   False
   FullScreenButton=   "False"
   HasBackColor    =   False
   Height          =   768
   ImplicitInstance=   True
   LiveResize      =   True
   MacProcID       =   0
   MaxHeight       =   32000
   MaximizeButton  =   False
   MaxWidth        =   32000
   MenuBar         =   504088575
   MenuBarVisible  =   True
   MinHeight       =   64
   MinimizeButton  =   True
   MinWidth        =   64
   Placement       =   2
   Resizeable      =   False
   Title           =   "ABCommanche"
   Visible         =   True
   Width           =   1024
   Begin Canvas Canvas1
      AcceptFocus     =   True
      AcceptTabs      =   False
      AutoDeactivate  =   True
      Backdrop        =   0
      DoubleBuffer    =   False
      Enabled         =   True
      EraseBackground =   True
      Height          =   768
      HelpTag         =   ""
      Index           =   -2147483648
      InitialParent   =   ""
      Left            =   0
      LockBottom      =   False
      LockedInPosition=   False
      LockLeft        =   False
      LockRight       =   False
      LockTop         =   False
      Scope           =   0
      TabIndex        =   0
      TabPanelIndex   =   0
      TabStop         =   True
      Top             =   0
      Transparent     =   True
      UseFocusRing    =   True
      Visible         =   True
      Width           =   1024
   End
End
#tag EndWindow

#tag WindowCode
	#tag Event
		Sub Open()
		  ' initialize the buffers
		  mBuffer = new Picture(1024,768,32)
		  mBuildBuffer = new Picture(VoxWidth, VoxHeight, 32)
		  mBuildBufferRGB = mBuildBuffer.RGBSurface
		  
		  ' create the chopper picture with the transparent areas
		  Chopper = new Picture(1024,768,32)
		  Chopper.Graphics.DrawPicture Comanche,0,0
		  Chopper.Mask.Graphics.DrawPicture ComancheMask,0,0
		  
		  ' load a map
		  CurrentMap = 1
		  LoadMap(CurrentMap)
		  
		End Sub
	#tag EndEvent


	#tag Method, Flags = &h0
		Sub LoadMap(MapID as integer)
		  dim x,y as Integer
		  'Initialize the position and the lookup/lookdown vars
		  xp = 512
		  yp = 800
		  hp = -50
		  vp = -100
		  
		  ' prebuild the tables holding the pixel data of the map and depthmap to speed things up
		  select case MapID
		  case 1
		    VoxelMapRGB = C1W.RGBSurface
		    for y = 0 to 1023
		      for x = 0 to 1023
		        VoxelHeight(y,x) = C1D.RGBSurface.Pixel(y,x).Red
		      next
		    next
		  case 2
		    VoxelMapRGB = C2W.RGBSurface
		    for y = 0 to 1023
		      for x = 0 to 1023
		        VoxelHeight(y,x) = C2D.RGBSurface.Pixel(y,x).Red
		      next
		    next
		  case 3
		    VoxelMapRGB = C3W.RGBSurface
		    for y = 0 to 1023
		      for x = 0 to 1023
		        VoxelHeight(y,x) = C3D.RGBSurface.Pixel(y,x).Red
		      next
		    next
		  case 4
		    VoxelMapRGB = C4W.RGBSurface
		    for y = 0 to 1023
		      for x = 0 to 1023
		        VoxelHeight(y,x) = C4D.RGBSurface.Pixel(y,x).Red
		      next
		    next
		  end select
		  
		  ' Draw!
		  Update
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Raycast(line as integer, x1 as double, y1 as double, x2 as double, y2 as double, d as double)
		  ' The magic! Raycast on the current line. 
		  
		  ' line: vertical line to compute
		  ' x1, y1: initial points on map for ray
		  ' x2, y2: final points on map for ray
		  ' d: correction parameter for the perspective
		  
		  dim dx,dy as Double
		  dx = x2 - x1
		  dy = y2 - y1
		  
		  dim r as Double = sqrt(dx * dx + dy * dy)
		  dx = dx / r
		  dy = dy / r
		  
		  dim yMin as Integer = 256
		  dim i,k as Integer
		  dim h as Integer
		  dim ci as Color
		  dim y3 as Double
		  dim z3 as Integer
		  
		  while i < r-20
		    i = i + 1
		    x1 = x1 + dx
		    y1 = y1 + dy
		    if x1 > 0 and y1 > 0 and x1 < 1024 and y1 < 1024 then
		      ' get the height
		      h = VoxelHeight(x1, y1)
		      ' get the color
		      ci = VoxelMapRGB.Pixel(x1, y1)
		      
		      ' calculate the ray depth. Do some perspective correction for the height.
		      h = 256 - h
		      h = h - 128 + hp
		      y3 = abs(d) * i
		      z3 = h / y3 * 100 - vp
		      
		      if (z3 < 0) then z3 = 0
		      if (z3 < voxHeight) then
		        k = z3
		        While k < ymin
		          ' draw a vertical line with the corresponding color with the height retrieved from the perspective correction
		          mBuildBufferRGB.Pixel(line, k) = ci
		          k = k + 1
		        wend
		      end if
		      
		      if (ymin > z3) then
		         ymin = z3
		      end if
		    end if
		  wend
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Update()
		  dim i as Integer
		  dim x3d, y3d, rotx, roty as Double
		  
		  ' draw the sky first on the end picture
		  mBuildBuffer.Graphics.DrawPicture Sky,0,0
		  
		  y3d = -VoxDepth
		  
		  ' update the view. Calculate the rotation and raycast every line
		  while i < VoxWidth
		    x3d = (i - VoxWidth / 2) * 1.5
		    
		    rotx =cos(angle) * x3d + sin(angle) * y3d
		    roty = -sin(angle) * x3d + cos(angle) * y3d
		    
		    raycast(i, xp, yp, rotx + xp, roty + yp, y3d / sqrt(x3d * x3d + y3d * y3d))
		    i = i + 1
		  wend
		  
		  ' draw the result on the end picture
		  mBuffer.Graphics.DrawPicture mBuildBuffer, 0,0, 1024, 512, 0,0,512,256
		  ' draw the chopper overlay
		  mBuffer.Graphics.DrawPicture Chopper,0,0
		  
		  ' refresh the canvas
		  Canvas1.Refresh(False)
		  
		End Sub
	#tag EndMethod


	#tag Property, Flags = &h0
		Angle As Double
	#tag EndProperty

	#tag Property, Flags = &h0
		Chopper As Picture
	#tag EndProperty

	#tag Property, Flags = &h0
		CurrentMap As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		hp As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		mBuffer As Picture
	#tag EndProperty

	#tag Property, Flags = &h0
		mBuildBuffer As Picture
	#tag EndProperty

	#tag Property, Flags = &h0
		mBuildBufferRGB As RGBSurface
	#tag EndProperty

	#tag Property, Flags = &h0
		VoxDepth As Integer = 400
	#tag EndProperty

	#tag Property, Flags = &h0
		VoxelHeight(1023,1023) As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		VoxelMapRGB As RGBSurface
	#tag EndProperty

	#tag Property, Flags = &h0
		VoxHeight As Integer = 256
	#tag EndProperty

	#tag Property, Flags = &h0
		VoxWidth As Integer = 512
	#tag EndProperty

	#tag Property, Flags = &h0
		vp As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		xp As Double
	#tag EndProperty

	#tag Property, Flags = &h0
		yp As Double
	#tag EndProperty


#tag EndWindowCode

#tag Events Canvas1
	#tag Event
		Sub Paint(g As Graphics, areas() As REALbasic.Rect)
		  ' draw the buffer to the canvas
		  if mBuffer <> nil then
		    g.DrawPicture mBuffer,0,0
		  end if
		End Sub
	#tag EndEvent
	#tag Event
		Function KeyDown(Key As String) As Boolean
		  '        Forward
		  '             8
		  '             |
		  'Left 4 -   - 6 Right
		  '             |
		  '             2
		  '          Back
		  
		  '           Up
		  '            7
		  '            |
		  '            1
		  '       Down
		  
		  '  Looking Down
		  '            9
		  '            |
		  '            3
		  '    Looking Up
		  
		  ' M: Cycle through maps
		  
		  select case key
		  case "4" ' left
		    angle = angle + 0.1
		  case "6" ' right
		    angle = angle - 0.1
		  case "8" ' forward
		    xp = xp - 2.0 * sin(angle)
		    yp = yp - 2.0 * cos(angle)
		  case "2" ' backward
		    xp = xp + 2.0 * sin(angle)
		    yp = yp + 2.0 * cos(angle)
		  case "7" ' up
		    hp = hp + 2
		  case "1" ' down
		    hp = hp - 2
		  case "9" ' look up
		    vp = vp + 2
		  case "3" ' look down
		    vp = vp - 2
		  case "m", "M" ' cycle through the maps
		    CurrentMap = CurrentMap + 1
		    if CurrentMap = 5 then CurrentMap = 1
		    LoadMap(CurrentMap)
		    Return true
		  end select
		  
		  ' update with the new params
		  Update
		  
		  Return true
		  
		End Function
	#tag EndEvent
#tag EndEvents
#tag ViewBehavior
	#tag ViewProperty
		Name="Angle"
		Group="Behavior"
		InitialValue="0"
		Type="Double"
	#tag EndViewProperty
	#tag ViewProperty
		Name="BackColor"
		Visible=true
		Group="Appearance"
		InitialValue="&hFFFFFF"
		Type="Color"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Backdrop"
		Visible=true
		Group="Appearance"
		Type="Picture"
		EditorType="Picture"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Chopper"
		Group="Behavior"
		InitialValue="0"
		Type="Picture"
	#tag EndViewProperty
	#tag ViewProperty
		Name="CloseButton"
		Visible=true
		Group="Appearance"
		InitialValue="True"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Composite"
		Visible=true
		Group="Appearance"
		InitialValue="False"
		Type="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="CurrentMap"
		Group="Behavior"
		InitialValue="0"
		Type="Integer"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Frame"
		Visible=true
		Group="Appearance"
		InitialValue="0"
		Type="Integer"
		EditorType="Enum"
		InheritedFrom="Window"
		#tag EnumValues
			"0 - Document"
			"1 - Movable Modal"
			"2 - Modal Dialog"
			"3 - Floating Window"
			"4 - Plain Box"
			"5 - Shadowed Box"
			"6 - Rounded Window"
			"7 - Global Floating Window"
			"8 - Sheet Window"
			"9 - Metal Window"
			"10 - Drawer Window"
			"11 - Modeless Dialog"
		#tag EndEnumValues
	#tag EndViewProperty
	#tag ViewProperty
		Name="FullScreen"
		Visible=true
		Group="Appearance"
		InitialValue="False"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="HasBackColor"
		Visible=true
		Group="Appearance"
		InitialValue="False"
		Type="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Height"
		Visible=true
		Group="Position"
		InitialValue="400"
		Type="Integer"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="hp"
		Group="Behavior"
		InitialValue="0"
		Type="Integer"
	#tag EndViewProperty
	#tag ViewProperty
		Name="ImplicitInstance"
		Visible=true
		Group="Appearance"
		InitialValue="True"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Interfaces"
		Visible=true
		Group="ID"
		Type="String"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="LiveResize"
		Visible=true
		Group="Appearance"
		InitialValue="True"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MacProcID"
		Visible=true
		Group="Appearance"
		InitialValue="0"
		Type="Integer"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MaxHeight"
		Visible=true
		Group="Position"
		InitialValue="32000"
		Type="Integer"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MaximizeButton"
		Visible=true
		Group="Appearance"
		InitialValue="False"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MaxWidth"
		Visible=true
		Group="Position"
		InitialValue="32000"
		Type="Integer"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="mBuffer"
		Group="Behavior"
		InitialValue="0"
		Type="Picture"
	#tag EndViewProperty
	#tag ViewProperty
		Name="mBuildBuffer"
		Group="Behavior"
		InitialValue="0"
		Type="Picture"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MenuBar"
		Visible=true
		Group="Appearance"
		Type="MenuBar"
		EditorType="MenuBar"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MenuBarVisible"
		Visible=true
		Group="Appearance"
		InitialValue="True"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MinHeight"
		Visible=true
		Group="Position"
		InitialValue="64"
		Type="Integer"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MinimizeButton"
		Visible=true
		Group="Appearance"
		InitialValue="True"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="MinWidth"
		Visible=true
		Group="Position"
		InitialValue="64"
		Type="Integer"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Name"
		Visible=true
		Group="ID"
		Type="String"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Placement"
		Visible=true
		Group="Position"
		InitialValue="0"
		Type="Integer"
		EditorType="Enum"
		InheritedFrom="Window"
		#tag EnumValues
			"0 - Default"
			"1 - Parent Window"
			"2 - Main Screen"
			"3 - Parent Window Screen"
			"4 - Stagger"
		#tag EndEnumValues
	#tag EndViewProperty
	#tag ViewProperty
		Name="Resizeable"
		Visible=true
		Group="Appearance"
		InitialValue="True"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Super"
		Visible=true
		Group="ID"
		Type="String"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Title"
		Visible=true
		Group="Appearance"
		InitialValue="Untitled"
		Type="String"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Visible"
		Visible=true
		Group="Appearance"
		InitialValue="True"
		Type="Boolean"
		EditorType="Boolean"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="VoxDepth"
		Group="Behavior"
		InitialValue="400"
		Type="Integer"
	#tag EndViewProperty
	#tag ViewProperty
		Name="VoxHeight"
		Group="Behavior"
		InitialValue="256"
		Type="Integer"
	#tag EndViewProperty
	#tag ViewProperty
		Name="VoxWidth"
		Group="Behavior"
		InitialValue="512"
		Type="Integer"
	#tag EndViewProperty
	#tag ViewProperty
		Name="vp"
		Group="Behavior"
		InitialValue="0"
		Type="Integer"
	#tag EndViewProperty
	#tag ViewProperty
		Name="Width"
		Visible=true
		Group="Position"
		InitialValue="600"
		Type="Integer"
		InheritedFrom="Window"
	#tag EndViewProperty
	#tag ViewProperty
		Name="xp"
		Group="Behavior"
		InitialValue="0"
		Type="Double"
	#tag EndViewProperty
	#tag ViewProperty
		Name="yp"
		Group="Behavior"
		InitialValue="0"
		Type="Double"
	#tag EndViewProperty
#tag EndViewBehavior
